Etiquette (Etiquette v0.1.1)
View SourceDefine packet specifications using Spec.
A packet consists of formatted data. Declaring a specification, it's possible to define the
structure of a packet and the different forms it can take. In essence, a packet is a sequence of
fields, so Etiquette.Spec can be used to import everything that is needed
to create a specification:
use Etiquette.Spec
Once Etiquette.Spec is used, the macros
packet and field are
imported.